// File: LocalSDABN.cpp
// Purpose: Implementation of the LocalSDABN class
// Remarks: This class is designed for LocalSDABN classification.
// Author: Ingrid Liu (ingrid@hkn.eecs.berkeley.edu)
// Last modified: 3/6/2008 4:52:21 PM

#include <cstdlib>
#include <iostream>
#include <algorithm>
#include "LocalSDABN.h"
#include "RealMatrix.h"
#include "LocalSDA.h"


using namespace std;

namespace idl
{

	bool LocalSDABN::classify(const SimData &data, vector<int> &results, RealMatrix &probMatrix)
	{
		// Check the neighborhood size: the neighborhood size must be
		// less than or equal to the training set size
		if (data.getTrainingSetSize() < parameters.neighborhoodSizes.back())
		{
			cerr << "Error: Neighborhood size too large" << endl;
			return false;
		}

		// Get the number of test samples
		unsigned int numTestSamples = data.getTestSetSize();
		results.resize(numTestSamples);

		vector<int> uniqueLabels; 
		data.getUniqueLabels(uniqueLabels);
		unsigned int nUniqueLabels = (unsigned int)uniqueLabels.size();
		RealMatrix probabilities;

		vector<int> dummyResults;
		probMatrix.zeros(numTestSamples,nUniqueLabels);
		
		for (vector<unsigned int>::const_iterator itrN = parameters.neighborhoodSizes.begin();
				itrN != parameters.neighborhoodSizes.end(); ++itrN)
		{
			LocalSDA::setNeighborhoodSize(*itrN);
			RealMatrix tmpP;
			LocalSDA::classify(data,dummyResults,tmpP);
			probMatrix += tmpP;
			
		}
	
		for (unsigned int i = 0; i < probMatrix.getRowNum(); ++i)
		{
			double sumRow = probMatrix.sumRow(i);
			for (unsigned int j = 0; j < probMatrix.getColumnNum(); ++j)
			{
				if (probMatrix(i,j) != 0)
					probMatrix(i,j) = probMatrix(i,j) / sumRow;
			}
		}

		for (unsigned int i = 0; i < numTestSamples; ++i)
		{
			results[i] = maxLabel(uniqueLabels,probMatrix.row(i));//lsdaModel.classify(simMat, labels, simVec, simDomain); 
		}			
		return true;
	}

	bool LocalSDABN::classify(const SimData &data, vector<int> &results)
	{
		RealMatrix probMatrix;
		return classify(data,results,probMatrix);
	}
	
	// Return the label with the largest probability
	int LocalSDABN::maxLabel(const vector<int> &labels, const RealMatrix &probabilities)
	{
		double tmpMaxP = -DBL_MAX;
		int tmpMaxLabel = -1;
		unsigned int tmpIdx = 0;
		for (unsigned int tmpIdx = 0; tmpIdx < probabilities.size()[1]; ++tmpIdx)
		{
			if (probabilities(0,tmpIdx) > tmpMaxP)
			{
				tmpMaxP = probabilities(0,tmpIdx);
				tmpMaxLabel = tmpIdx;
			}
		}
		//
		//for (unsigned int j = 0; j < probabilities.size()[1]; ++j)
		//{
		//	std::cout << probabilities(j) << " ";
		//}
		//std::cout <<std::endl;
		//std::cout <<"label: " << labels[tmpMaxLabel] <<std::endl;
		//std::cout << tmpMaxLabel;
		return labels[tmpMaxLabel];
	}	

}
