// File: main.cpp
// Purpose: Entry point to the program
// Author: Yihua Chen (yhchen@u.washington.edu)
// Last modified: 3/6/2008 4:51:06 PM

#include <cstdlib>
#include <ctime>
#include <iostream>
#include "CmRandomNumberGenerator.h" // Mersenne Twister (singleton) header file
#include "ConfigParser.h"
#include "SimTest.h"

/**
 * The main() function.
 * Entry point to the program.
 * @param[in]	argc	number of command line arguments
 * @param[in]	argv	array of strings for command line arguments
 * @return		0 if success; 1 if failure
 */
int main(int argc, char *argv[])
{
	// Seed the random number generator
	haruki::CmRandomNumberGenerator *pRandomNumberGenerator =
		haruki::CmRandomNumberGenerator::getInstance();
	pRandomNumberGenerator->changeSeed(static_cast<unsigned long>(time(NULL)));

	idl::ConfigParser config;

	// Parse the command line arguments
	if (!config.parseCommandLine(argc, argv))
	{
		return EXIT_FAILURE;
	}

	idl::SimTest test;

	// Start the test according to the configurations
	if (!test.start(config))
	{
		return EXIT_FAILURE;
	}

	std::cout << "Job completed" << std::endl;
	
	return EXIT_SUCCESS;
}
